@@ -51,6 +51,7 @@ cmd_show_neighbors(struct lldpctl_conn_t *conn, struct writer *w,
  *
  * The environment will contain the following keys:
  *  - C{ports} list of ports we want to restrict showing.
+ *  - C{summary} summary of stats
  */
 static int
 cmd_show_interface_stats(struct lldpctl_conn_t *conn, struct writer *w,
@@ -60,6 +61,8 @@ cmd_show_interface_stats(struct lldpctl_conn_t *conn, struct writer *w,
 	if (cmdenv_get(env, "ports"))
 		log_debug("lldpctl", "restrict to the following ports: %s",
 		    cmdenv_get(env, "ports"));
+	if (cmdenv_get(env, "summary"))
+		log_debug("lldpctl", "show summary of stats accross ports");
 
 	display_interfaces_stats(conn, w, env);
 
@@ -86,6 +89,11 @@ cmd_show_configuration(struct lldpctl_conn_t *conn, struct writer *w,
 	return 1;
 }
 
+struct watcharg {
+	struct cmd_env *env;
+	struct writer *w;
+};
+
 /**
  * Callback for the next function to display a new neighbor.
  */
@@ -96,8 +104,9 @@ watchcb(lldpctl_conn_t *conn,
     lldpctl_atom_t *neighbor,
     void *data)
 {
-	struct cmd_env *env = data;
-	struct writer *w = (struct writer *)cmdenv_get(env, "writer");
+	struct watcharg *wa = data;
+	struct cmd_env *env = wa->env;
+	struct writer *w = wa->w;
 	const char *interfaces = cmdenv_get(env, "ports");
 	if (interfaces && !contains(interfaces, lldpctl_atom_get_str(interface,
 		    lldpctl_k_interface_name)))
@@ -130,14 +139,16 @@ cmd_watch_neighbors(struct lldpctl_conn_t *conn, struct writer *w,
     struct cmd_env *env, void *arg)
 {
 	int watch = 1;
+	struct watcharg wa = {
+		.env = env,
+		.w = w
+	};
 	log_debug("lldpctl", "watch for neighbor changes");
-	if (lldpctl_watch_callback(conn, watchcb, env) < 0) {
+	if (lldpctl_watch_callback(conn, watchcb, &wa) < 0) {
 		log_warnx("lldpctl", "unable to watch for neighbors. %s",
 		    lldpctl_last_strerror(conn));
 		return 0;
 	}
-	cmdenv_put(env, "writer", (const char*)w); /* Hackish, but we really
-						    * don't care. */
 	while (watch) {
 		if (lldpctl_watch(conn) < 0) {
 			log_warnx("lldpctl", "unable to watch for neighbors. %s",
@@ -176,6 +187,18 @@ register_common_commands(struct cmd_node *root)
 	cmd_restrict_ports(root);
 }
 
+/**
+ * Register sub command summary
+ */
+void
+register_summary_command(struct cmd_node *root)
+{
+	commands_new(root,
+			"summary",
+			"With less details",
+			cmd_check_no_detailed_nor_summary, cmd_store_env_and_pop, "summary");
+}
+
 /**
  * Register subcommands to `show`
  *
@@ -216,6 +239,7 @@ register_commands_show(struct cmd_node *root)
 	    NULL, cmd_show_interface_stats, NULL);
 
 	cmd_restrict_ports(stats);
+	register_summary_command(stats);
 
 	/* Register "show configuration" and "show running-configuration" */
 	commands_new(
