@@ -33,6 +33,10 @@
 #ifndef _COMPAT_H
 #define _COMPAT_H
 
+#if HAVE_CONFIG_H
+#  include <config.h>
+#endif
+
 #include <stdio.h>
 #include <stddef.h>
 #ifdef HAVE_LIBBSD
@@ -49,6 +53,10 @@ size_t	strlcpy(char *, const char *, size_t);
 size_t	strnlen(const char *, size_t);
 #endif
 
+#if !HAVE_STRNDUP
+char	*strndup(const char *, size_t);
+#endif
+
 #if !HAVE_FGETLN
 char *fgetln(FILE *, size_t *);
 #endif
