@@ -125,7 +125,6 @@ ctl_msg_send_unserialized(uint8_t **output_buffer, size_t *output_len,
     enum hmsg_type type,
     void *t, struct marshal_info *mi)
 {
-	struct hmsg_header hdr;
 	ssize_t len = 0, newlen;
 	void *buffer = NULL;
 
@@ -156,9 +155,11 @@ ctl_msg_send_unserialized(uint8_t **output_buffer, size_t *output_len,
 		}
 		*output_buffer = new;
 	}
-	memset(&hdr, 0, sizeof(struct hmsg_header));
-	hdr.type = type;
-	hdr.len  = len;
+
+	struct hmsg_header hdr = {
+		.type = type,
+		.len = len
+	};
 	memcpy(*output_buffer + *output_len, &hdr, sizeof(struct hmsg_header));
 	if (t)
 		memcpy(*output_buffer + *output_len + sizeof(struct hmsg_header), buffer, len);
