@@ -16,11 +16,13 @@
  */
 
 #include "lldpd.h"
+#include "trace.h"
 
 #include <unistd.h>
 #include <signal.h>
 #include <errno.h>
 #include <time.h>
+#include <fcntl.h>
 #include <event2/event.h>
 #include <event2/bufferevent.h>
 #include <event2/buffer.h>
@@ -104,7 +106,7 @@ levent_snmp_add_fd(struct lldpd *cfg, int fd)
 		log_warn("event", "unable to allocate memory for new SNMP event");
 		return;
 	}
-	evutil_make_socket_nonblocking(fd);
+	levent_make_socket_nonblocking(fd);
 	if ((snmpfd->ev = event_new(base, fd,
 				    EV_READ | EV_PERSIST,
 				    levent_snmp_read,
@@ -358,7 +360,7 @@ levent_ctl_accept(evutil_socket_t fd, short what, void *arg)
 		goto accept_failed;
 	}
 	client->cfg = cfg;
-	evutil_make_socket_nonblocking(s);
+	levent_make_socket_nonblocking(s);
 	TAILQ_INSERT_TAIL(&lldpd_clients, client, next);
 	if ((client->bev = bufferevent_socket_new(cfg->g_base, s,
 		    BEV_OPT_CLOSE_ON_FREE)) == NULL) {
@@ -458,7 +460,7 @@ levent_init(struct lldpd *cfg)
 	/* Setup unix socket */
 	log_debug("event", "register Unix socket");
 	TAILQ_INIT(&lldpd_clients);
-	evutil_make_socket_nonblocking(cfg->g_ctl);
+	levent_make_socket_nonblocking(cfg->g_ctl);
 	if ((cfg->g_ctl_event = event_new(cfg->g_base, cfg->g_ctl,
 		    EV_READ|EV_PERSIST, levent_ctl_accept, cfg)) == NULL)
 		fatalx("unable to setup control socket event");
@@ -484,23 +486,16 @@ levent_loop(struct lldpd *cfg)
 {
 	levent_init(cfg);
 	lldpd_loop(cfg);
+#ifdef USE_SNMP
+	if (cfg->g_snmp) levent_snmp_update(cfg);
+#endif
 
 	/* libevent loop */
 	do {
+		TRACE(LLDPD_EVENT_LOOP());
 		if (event_base_got_break(cfg->g_base) ||
 		    event_base_got_exit(cfg->g_base))
 			break;
-#ifdef USE_SNMP
-		if (cfg->g_snmp) {
-			/* We don't use delegated requests (request
-			   whose answer is delayed). However, we keep
-			   the call here in case we use it some
-			   day. We don't call run_alarms() here. We do
-			   it on timeout only. */
-			netsnmp_check_outstanding_agent_requests();
-			levent_snmp_update(cfg);
-		}
-#endif
 	} while (event_base_loop(cfg->g_base, EVLOOP_ONCE) == 0);
 
 #ifdef USE_SNMP
@@ -547,7 +542,7 @@ levent_hardware_add_fd(struct lldpd_hardware *hardware, int fd)
 		    hardware->h_ifname);
 		return;
 	}
-	evutil_make_socket_nonblocking(fd);
+	levent_make_socket_nonblocking(fd);
 	if ((hfd->ev = event_new(hardware->h_cfg->g_base, fd,
 		    EV_READ | EV_PERSIST,
 		    levent_hardware_recv,
@@ -625,6 +620,7 @@ levent_iface_recv(evutil_socket_t fd, short what, void *arg)
 	/* Schedule local port update. We don't run it right away because we may
 	 * receive a batch of events like this. */
 	struct timeval one_sec = {1, 0};
+	TRACE(LLDPD_INTERFACES_NOTIFICATION());
 	log_debug("event",
 	    "received notification change, schedule an update of all interfaces in one second");
 	if (cfg->g_iface_timer_event == NULL) {
@@ -647,7 +643,7 @@ levent_iface_subscribe(struct lldpd *cfg, int socket)
 {
 	log_debug("event", "subscribe to interface changes from socket %d",
 	    socket);
-	evutil_make_socket_nonblocking(socket);
+	levent_make_socket_nonblocking(socket);
 	cfg->g_iface_event = event_new(cfg->g_base, socket,
 	    EV_READ | EV_PERSIST, levent_iface_recv, cfg);
 	if (cfg->g_iface_event == NULL) {
@@ -763,3 +759,19 @@ levent_schedule_pdu(struct lldpd_hardware *hardware)
 		return;
 	}
 }
+
+int
+levent_make_socket_nonblocking(int fd)
+{
+	int flags;
+	if ((flags = fcntl(fd, F_GETFL, NULL)) < 0) {
+		log_warn("event", "fcntl(%d, F_GETFL)", fd);
+		return -1;
+	}
+	if (flags & O_NONBLOCK) return 0;
+	if (fcntl(fd, F_SETFL, flags | O_NONBLOCK) == -1) {
+		log_warn("event", "fcntl(%d, F_SETFL)", fd);
+		return -1;
+	}
+	return 0;
+}
