@@ -229,6 +229,12 @@ iflinux_is_bond(struct lldpd *cfg,
     struct interfaces_device_list *interfaces,
     struct interfaces_device *master)
 {
+	/* Shortcut if we detect the new team driver. Upper and lower links
+	 * should already be set with netlink in this case.  */
+	if (master->driver && !strcmp(master->driver, "team")) {
+		return 1;
+	}
+
 	struct ifreq ifr = {};
 	struct ifbond ifb = {};
 	strlcpy(ifr.ifr_name, master->name, sizeof(ifr.ifr_name));
@@ -272,7 +278,7 @@ iflinux_get_permanent_mac(struct lldpd *cfg,
 	char path[SYSFS_PATH_MAX];
 	char line[100];
 
-	if ((master = iface->upper) == NULL)
+	if ((master = iface->upper) == NULL || master->type != IFACE_BOND_T)
 		return;
 
 	log_debug("interfaces", "get MAC address for %s",
@@ -465,27 +471,6 @@ iface_bond_init(struct lldpd *cfg, struct lldpd_hardware *hardware)
 	return 0;
 }
 
-static int
-iface_bond_send(struct lldpd *cfg, struct lldpd_hardware *hardware,
-    char *buffer, size_t size)
-{
-	/* With bonds, we have duplicate MAC address on different physical
-	 * interfaces. We need to alter the source MAC address when we send on
-	 * an inactive slave. To avoid any future problem, we always set the
-	 * source MAC address to 0. */
-	log_debug("interfaces", "send PDU to bonded device %s",
-	    hardware->h_ifname);
-	if (size < 2 * ETHER_ADDR_LEN) {
-		log_warnx("interfaces",
-		    "packet to send on %s is too small!",
-		    hardware->h_ifname);
-		return 0;
-	}
-	memset(buffer + ETHER_ADDR_LEN, 0, ETHER_ADDR_LEN);
-	return write(hardware->h_sendfd,
-	    buffer, size);
-}
-
 static int
 iface_bond_recv(struct lldpd *cfg, struct lldpd_hardware *hardware,
     int fd, char *buffer, size_t size)
@@ -535,7 +520,7 @@ iface_bond_close(struct lldpd *cfg, struct lldpd_hardware *hardware)
 }
 
 struct lldpd_ops bond_ops = {
-	.send = iface_bond_send,
+	.send = iflinux_eth_send,
 	.recv = iface_bond_recv,
 	.cleanup = iface_bond_close,
 };
@@ -579,7 +564,8 @@ iflinux_handle_bond(struct lldpd *cfg, struct interfaces_device_list *interfaces
 				continue;
 			}
 			hardware->h_ops = &bond_ops;
-			TAILQ_INSERT_TAIL(&cfg->g_hardware, hardware, h_entries);
+			hardware->h_mangle = 1;
+			interfaces_helper_add_hardware(cfg, hardware);
 		} else {
 			if (hardware->h_flags) continue; /* Already seen this time */
 			bmaster = hardware->h_data;
@@ -683,8 +669,6 @@ iflinux_add_bond(struct lldpd *cfg,
 			    "interface %s is a bond",
 			    iface->name);
 			iface->type |= IFACE_BOND_T;
-			iflinux_get_permanent_mac(cfg,
-			    interfaces, iface);
 		}
 	}
 }
@@ -765,6 +749,9 @@ iflinux_add_physical(struct lldpd *cfg,
 			continue;
 		}
 
+		/* Get the real MAC address (for example, if the interface is enslaved) */
+		iflinux_get_permanent_mac(cfg, interfaces, iface);
+
 		log_debug("interfaces",
 		    "%s is a physical interface",
 		    iface->name);
