@@ -16,6 +16,7 @@
  */
 
 #include "lldpd.h"
+#include "trace.h"
 
 #include <stddef.h>
 #include <unistd.h>
@@ -495,6 +496,14 @@ interfaces_helper_port_name_desc(struct lldpd_hardware *hardware,
 	port->p_descr = strdup(iface->alias);
 }
 
+void
+interfaces_helper_add_hardware(struct lldpd *cfg,
+    struct lldpd_hardware *hardware)
+{
+	TRACE(LLDPD_INTERFACES_NEW(hardware->h_ifname));
+	TAILQ_INSERT_TAIL(&cfg->g_hardware, hardware, h_entries);
+}
+
 void
 interfaces_helper_physical(struct lldpd *cfg,
     struct interfaces_device_list *interfaces,
@@ -529,7 +538,9 @@ interfaces_helper_physical(struct lldpd *cfg,
 				continue;
 			}
 			hardware->h_ops = ops;
-			TAILQ_INSERT_TAIL(&cfg->g_hardware, hardware, h_entries);
+			hardware->h_mangle = (iface->upper &&
+			    iface->upper->type & IFACE_BOND_T);
+			interfaces_helper_add_hardware(cfg, hardware);
 		} else {
 			if (hardware->h_flags) continue; /* Already seen this time */
 			lldpd_port_cleanup(&hardware->h_lport, 0);
@@ -556,3 +567,47 @@ interfaces_helper_physical(struct lldpd *cfg,
 #endif
 	}
 }
+
+/**
+ * Send the packet using the hardware function. Optionnaly mangle the MAC address.
+ *
+ * With bonds, we have duplicate MAC address on different physical
+ * interfaces. We need to alter the source MAC address when we send on an
+ * inactive slave. The `h_mangle` flah is used to know if we need to do
+ * something like that.
+ */
+int
+interfaces_send_helper(struct lldpd *cfg,
+    struct lldpd_hardware *hardware,
+    char *buffer, size_t size)
+{
+	if (size < 2 * ETHER_ADDR_LEN) {
+		log_warnx("interfaces",
+		    "packet to send on %s is too small!",
+		    hardware->h_ifname);
+		return 0;
+	}
+	if (hardware->h_mangle) {
+#define MAC_UL_ADMINISTERED_BIT_MASK 0x02
+		char *src_mac = buffer + ETHER_ADDR_LEN;
+		char arbitrary[] = { 0x00, 0x60, 0x08, 0x69, 0x97, 0xef};
+
+		switch (cfg->g_config.c_bond_slave_src_mac_type) {
+		case LLDP_BOND_SLAVE_SRC_MAC_TYPE_LOCALLY_ADMINISTERED:
+			if (*src_mac & MAC_UL_ADMINISTERED_BIT_MASK) {
+				/* If locally administered bit already set,
+				 * use zero mac
+				 */
+				memset(src_mac, 0, ETHER_ADDR_LEN);
+				break;
+			}
+		case LLDP_BOND_SLAVE_SRC_MAC_TYPE_FIXED:
+			memcpy(src_mac, arbitrary, ETHER_ADDR_LEN);
+			break;
+		case LLDP_BOND_SLAVE_SRC_MAC_TYPE_ZERO:
+			memset(src_mac, 0, ETHER_ADDR_LEN);
+			break;
+		}
+	}
+	return hardware->h_ops->send(cfg, hardware, buffer, size);
+}
