@@ -16,6 +16,7 @@
  */
 
 #include "lldpd.h"
+#include "trace.h"
 
 #include <stdio.h>
 #include <unistd.h>
@@ -275,6 +276,9 @@ static void
 notify_clients_deletion(struct lldpd_hardware *hardware,
     struct lldpd_port *rport)
 {
+	TRACE(LLDPD_NEIGHBOR_DELETE(hardware->h_ifname,
+		rport->p_chassis->c_name,
+		rport->p_descr));
 	levent_ctl_notify(hardware->h_ifname, NEIGHBOR_CHANGE_DELETED,
 	    rport);
 #ifdef USE_SNMP
@@ -334,11 +338,12 @@ lldpd_cleanup(struct lldpd *cfg)
 	     hardware = hardware_next) {
 		hardware_next = TAILQ_NEXT(hardware, h_entries);
 		if (!hardware->h_flags) {
+			TRACE(LLDPD_INTERFACES_DELETE(hardware->h_ifname));
 			TAILQ_REMOVE(&cfg->g_hardware, hardware, h_entries);
-			lldpd_remote_cleanup(hardware, NULL);
+			lldpd_remote_cleanup(hardware, notify_clients_deletion, 1);
 			lldpd_hardware_cleanup(cfg, hardware);
 		} else
-			lldpd_remote_cleanup(hardware, notify_clients_deletion);
+			lldpd_remote_cleanup(hardware, notify_clients_deletion, 0);
 	}
 
 	log_debug("localchassis", "cleanup all chassis");
@@ -482,6 +487,11 @@ lldpd_decode(struct lldpd *cfg, char *frame, int s,
 		    hardware->h_ifname);
 		return;
 	}
+	TRACE(LLDPD_FRAME_DECODED(
+		    hardware->h_ifname,
+		    cfg->g_protocols[i].name,
+		    chassis->c_name,
+		    port->p_descr));
 
 	/* Do we already have the same MSAP somewhere? */
 	int count = 0;
@@ -577,11 +587,25 @@ lldpd_decode(struct lldpd *cfg, char *frame, int s,
 	/* Notify */
 	log_debug("decode", "send notifications for changes on %s",
 	    hardware->h_ifname);
-	i = oport?NEIGHBOR_CHANGE_UPDATED:NEIGHBOR_CHANGE_ADDED;
-	levent_ctl_notify(hardware->h_ifname, i, port);
+	if (oport) {
+		TRACE(LLDPD_NEIGHBOR_UPDATE(hardware->h_ifname,
+			chassis->c_name,
+			port->p_descr,
+			i));
+		levent_ctl_notify(hardware->h_ifname, NEIGHBOR_CHANGE_UPDATED, port);
 #ifdef USE_SNMP
-	agent_notify(hardware, i, port);
+		agent_notify(hardware, NEIGHBOR_CHANGE_UPDATED, port);
 #endif
+	} else {
+		TRACE(LLDPD_NEIGHBOR_NEW(hardware->h_ifname,
+			chassis->c_name,
+			port->p_descr,
+			i));
+		levent_ctl_notify(hardware->h_ifname, NEIGHBOR_CHANGE_ADDED, port);
+#ifdef USE_SNMP
+		agent_notify(hardware, NEIGHBOR_CHANGE_ADDED, port);
+#endif
+	}
 
 #ifdef ENABLE_LLDPMED
 	if (!oport && port->p_chassis->c_med_type) {
@@ -851,6 +875,7 @@ lldpd_recv(struct lldpd *cfg, struct lldpd_hardware *hardware, int fd)
 	hardware->h_rx_cnt++;
 	log_debug("receive", "decode received frame on %s",
 	    hardware->h_ifname);
+	TRACE(LLDPD_FRAME_RECEIVED(hardware->h_ifname, buffer, (size_t)n));
 	lldpd_decode(cfg, buffer, n, hardware);
 	lldpd_hide_all(cfg); /* Immediatly hide */
 	lldpd_count_neighbors(cfg);
@@ -887,6 +912,8 @@ lldpd_send(struct lldpd_hardware *hardware)
 				continue;
 			if (port->p_protocol ==
 			    cfg->g_protocols[i].mode) {
+				TRACE(LLDPD_FRAME_SEND(hardware->h_ifname,
+					cfg->g_protocols[i].name));
 				log_debug("send", "send PDU on %s with protocol %s",
 				    hardware->h_ifname,
 				    cfg->g_protocols[i].name);
@@ -903,6 +930,8 @@ lldpd_send(struct lldpd_hardware *hardware)
 		 * available protocol. */
 		for (i = 0; cfg->g_protocols[i].mode != 0; i++) {
 			if (!cfg->g_protocols[i].enabled) continue;
+			TRACE(LLDPD_FRAME_SEND(hardware->h_ifname,
+				cfg->g_protocols[i].name));
 			log_debug("send", "fallback to protocol %s for %s",
 			    cfg->g_protocols[i].name, hardware->h_ifname);
 			cfg->g_protocols[i].send(cfg,
@@ -1025,6 +1054,7 @@ lldpd_update_localports(struct lldpd *cfg)
 	TAILQ_FOREACH(hardware, &cfg->g_hardware, h_entries)
 	    hardware->h_flags = 0;
 
+	TRACE(LLDPD_INTERFACES_UPDATE());
 	interfaces_update(cfg);
 	lldpd_cleanup(cfg);
 	lldpd_reset_timer(cfg);
@@ -1061,7 +1091,7 @@ lldpd_exit(struct lldpd *cfg)
 	     hardware = hardware_next) {
 		hardware_next = TAILQ_NEXT(hardware, h_entries);
 		log_debug("main", "cleanup interface %s", hardware->h_ifname);
-		lldpd_remote_cleanup(hardware, NULL);
+		lldpd_remote_cleanup(hardware, NULL, 1);
 		lldpd_hardware_cleanup(cfg, hardware);
 	}
 }
@@ -1072,7 +1102,7 @@ lldpd_exit(struct lldpd *cfg)
  * @return PID of running lldpcli or -1 if error.
  */
 static pid_t
-lldpd_configure(int debug, const char *path)
+lldpd_configure(int debug, const char *path, const char *ctlname)
 {
 	pid_t lldpcli = fork();
 	int devnull;
@@ -1095,6 +1125,7 @@ lldpd_configure(int debug, const char *path)
 
 			log_debug("main", "invoke %s %s", path, sdebug);
 			if (execl(path, "lldpcli", sdebug,
+				"-u", ctlname,
 				"-c", SYSCONFDIR "/lldpd.conf",
 				"-c", SYSCONFDIR "/lldpd.d",
 				"resume",
@@ -1217,7 +1248,7 @@ lldpd_started_by_systemd()
 #endif
 
 int
-lldpd_main(int argc, char *argv[])
+lldpd_main(int argc, char *argv[], char *envp[])
 {
 	struct lldpd *cfg;
 	struct lldpd_chassis *lchassis;
@@ -1253,6 +1284,10 @@ lldpd_main(int argc, char *argv[])
 
 	saved_argv = argv;
 
+#if HAVE_SETPROCTITLE_INIT
+	setproctitle_init(argc, argv, envp);
+#endif
+
 	/*
 	 * Get and parse command line options
 	 */
@@ -1414,7 +1449,7 @@ lldpd_main(int argc, char *argv[])
 	/* Configuration with lldpcli */
 	if (lldpcli) {
 		log_debug("main", "invoking lldpcli for configuration");
-		if (lldpd_configure(debug, lldpcli) == -1)
+		if (lldpd_configure(debug, lldpcli, ctlname) == -1)
 			fatal("main", "unable to spawn lldpcli");
 	}
 
@@ -1428,7 +1463,7 @@ lldpd_main(int argc, char *argv[])
 		if (daemon(0, 0) != 0)
 			fatal("main", "failed to detach daemon");
 		if ((pid = open(LLDPD_PID_FILE,
-			    O_TRUNC | O_CREAT | O_WRONLY, 0644)) == -1)
+			    O_TRUNC | O_CREAT | O_WRONLY, 0666)) == -1)
 			fatal("main", "unable to open pid file " LLDPD_PID_FILE);
 		if (asprintf(&spid, "%d\n", getpid()) == -1)
 			fatal("main", "unable to create pid file " LLDPD_PID_FILE);
@@ -1476,6 +1511,8 @@ lldpd_main(int argc, char *argv[])
 	cfg->g_snmp = snmp;
 	cfg->g_snmp_agentx = agentx;
 #endif /* USE_SNMP */
+	cfg->g_config.c_bond_slave_src_mac_type = \
+	    LLDP_BOND_SLAVE_SRC_MAC_TYPE_FIXED;
 
 	/* Get ioctl socket */
 	log_debug("main", "get an ioctl socket");
