@@ -21,6 +21,7 @@
  * case, error can be non fatal. */
 
 #include "lldpd.h"
+#include "trace.h"
 
 #include <stdio.h>
 #include <unistd.h>
@@ -57,7 +58,9 @@
 #ifdef HAVE_NETDB_H
 #  include <netdb.h>
 #endif
-#include <resolv.h>
+#ifdef HAVE_RESOLV_H
+#  include <resolv.h>
+#endif
 
 static int monitored = -1;		/* Child */
 
@@ -215,6 +218,7 @@ asroot_iface_init()
 	must_read(&name, sizeof(name));
 	name[sizeof(name) - 1] = '\0';
 
+	TRACE(LLDPD_PRIV_INTERFACE_INIT(name));
 	rc = asroot_iface_init_os(ifindex, name, &fd);
 	must_write(&rc, sizeof(rc));
 	if (rc == 0 && fd >=0) send_fd(fd);
@@ -274,6 +278,7 @@ asroot_iface_description()
 
 	must_read(description, len);
 	description[len] = 0;
+	TRACE(LLDPD_PRIV_INTERFACE_DESCRIPTION(name, description));
 	rc = asroot_iface_description_os(name, description);
 	must_write(&rc, sizeof(rc));
 }
@@ -337,12 +342,16 @@ static struct dispatch_actions actions[] = {
 
 /* Main loop, run as root */
 static void
-priv_loop()
+priv_loop(int remote)
 {
 	enum priv_cmd cmd;
 	struct dispatch_actions *a;
 
 	setproctitle("monitor");
+#ifdef USE_SECCOMP
+	if (priv_seccomp_init(remote, monitored) != 0)
+	   fatal("privsep", "cannot continue without seccomp setup");
+#endif
 	while (!may_read(&cmd, sizeof(enum priv_cmd))) {
 		for (a = actions; a->function != NULL; a++) {
 			if (cmd == a->msg) {
@@ -426,6 +435,84 @@ sig_chld(int sig)
 }
 
 /* Initialization */
+#define LOCALTIME "/etc/localtime"
+static void
+priv_setup_chroot(const char *chrootdir)
+{
+	/* Create chroot if it does not exist */
+	struct stat schroot;
+	if (stat(chrootdir, &schroot) == -1) {
+		if (errno != ENOENT)
+			fatal("privsep", "chroot directory does not exist");
+		if (mkdir(chrootdir, 0755) == -1)
+			fatal("privsep", "unable to create chroot directory");
+		log_info("privsep", "created chroot directory %s",
+		    chrootdir);
+	}
+
+	/* Check if /etc/localtime exists in chroot or outside chroot */
+	char path[1024];
+	if (snprintf(path, sizeof(path),
+		"%s" LOCALTIME, chrootdir) >= sizeof(path))
+		return;
+	if (stat(path, &schroot) != -1 ||
+	    stat(LOCALTIME, &schroot) == -1) return;
+
+	/* Prepare copy of /etc/localtime */
+	path[strlen(chrootdir) + 4] = '\0';
+	if (stat(path, &schroot) == -1) {
+		if (errno != ENOENT) return;
+		if (mkdir(path, 0755) == -1) {
+			log_warn("privsep", "unable to create %s directory",
+			    path);
+			return;
+		}
+	}
+	path[strlen(chrootdir) + 4] = '/';
+
+	/* Do copy */
+	int source = -1, destination = -1;
+	char buffer[1024];
+	ssize_t n;
+	if ((source = open(LOCALTIME, O_RDONLY)) == -1) {
+		log_warn("privsep", "cannot read " LOCALTIME);
+		return;
+	}
+	mode_t old = umask(S_IWGRP | S_IWOTH);
+	if ((destination = open(path,
+		    O_WRONLY | O_CREAT | O_TRUNC, 0666)) == -1) {
+		log_warn("privsep", "cannot create %s", path);
+		close(source);
+		umask(old);
+		return;
+	}
+	umask(old);
+	while ((n = read(source, buffer, sizeof(buffer))) > 0) {
+		ssize_t nw, left = n;
+		char *p = buffer;
+		while (left > 0) {
+			if ((nw = write(destination, p, left)) == -1) {
+				if (errno == EINTR) continue;
+				log_warn("privsep", "cannot write to %s", path);
+				close(source);
+				close(destination);
+				unlink(path);
+				return;
+			}
+			left -= nw;
+			p += nw;
+		}
+	}
+	if (n == -1) {
+		log_warn("privsep", "cannot read " LOCALTIME);
+		unlink(path);
+	} else {
+		log_info("privsep", LOCALTIME " copied to chroot");
+	}
+	close(source);
+	close(destination);
+}
+
 void
 priv_init(const char *chrootdir, int ctl, uid_t uid, gid_t gid)
 {
@@ -447,15 +534,7 @@ priv_init(const char *chrootdir, int ctl, uid_t uid, gid_t gid)
 		if (RUNNING_ON_VALGRIND)
 			log_warnx("privsep", "running on valgrind, keep privileges");
 		else {
-			struct stat schroot;
-			if (stat(chrootdir, &schroot) == -1) {
-				if (errno != ENOENT)
-					fatal("privsep", "chroot directory does not exist");
-				if (mkdir(chrootdir, 0755) == -1)
-					fatal("privsep", "unable to create chroot directory");
-				log_info("privsep", "created chroot directory %s",
-				    chrootdir);
-			}
+			priv_setup_chroot(chrootdir);
 			if (chroot(chrootdir) == -1)
 				fatal("privsep", "unable to chroot");
 			if (chdir("/") != 0)
@@ -490,16 +569,26 @@ priv_init(const char *chrootdir, int ctl, uid_t uid, gid_t gid)
 		if (atexit(priv_exit) != 0)
 			fatal("privsep", "unable to set exit function");
 
-		signal(SIGALRM, sig_pass_to_chld);
-		signal(SIGTERM, sig_pass_to_chld);
-		signal(SIGHUP, sig_pass_to_chld);
-		signal(SIGINT, sig_pass_to_chld);
-		signal(SIGQUIT, sig_pass_to_chld);
-		signal(SIGCHLD, sig_chld);
+		/* Install signal handlers */
+		const struct sigaction pass_to_child = {
+			.sa_handler = sig_pass_to_chld,
+			.sa_flags = SA_RESTART
+		};
+		sigaction(SIGALRM, &pass_to_child, NULL);
+		sigaction(SIGTERM, &pass_to_child, NULL);
+		sigaction(SIGHUP,  &pass_to_child, NULL);
+		sigaction(SIGINT,  &pass_to_child, NULL);
+		sigaction(SIGQUIT, &pass_to_child, NULL);
+		const struct sigaction child = {
+			.sa_handler = sig_chld,
+			.sa_flags = SA_RESTART
+		};
+		sigaction(SIGCHLD, &child, NULL);
+
                 if (waitpid(monitored, &status, WNOHANG) != 0)
                         /* Child is already dead */
                         _exit(1);
-		priv_loop();
+		priv_loop(pair[1]);
 		exit(0);
 	}
 }
