@@ -583,7 +583,7 @@ lldpctl_atom_t *lldpctl_get_port(lldpctl_atom_t *port);
  */
 typedef enum {
 	lldpctl_k_config_tx_interval, /**< `(I,WO)` Transmit interval. When set to -1, it is meant to transmit now. */
-	lldpctl_k_config_receiveonly, /**< `(I,WO)` Receive only mode */
+	lldpctl_k_config_receiveonly, /**< `(I)` Receive only mode */
 	lldpctl_k_config_mgmt_pattern, /**< `(S)` Pattern to choose the management address */
 	lldpctl_k_config_iface_pattern, /**< `(S,WO)` Pattern of enabled interfaces */
 	lldpctl_k_config_cid_pattern,	/**< `(S)` Interface pattern to choose the chassis ID */
@@ -594,6 +594,7 @@ typedef enum {
 	lldpctl_k_config_paused,	      /**< `(I)` lldpd is paused */
 	lldpctl_k_config_fast_start_enabled, /**< `(I,WO)` Is fast start enabled */
 	lldpctl_k_config_fast_start_interval, /**< `(I,WO)` Start fast transmit interval */
+	lldpctl_k_config_ifdescr_update, /**< `(I,WO)` Enable or disable setting interface description */
 
 	lldpctl_k_interface_name = 1000, /**< `(S)` The interface name. */
 
@@ -707,6 +708,7 @@ typedef enum {
 	lldpctl_k_insert_cnt,	/**< `(I)` insert cnt. Only works for a local port. */
 	lldpctl_k_delete_cnt,	/**< `(I)` delete cnt. Only works for a local port. */
 	lldpctl_k_config_tx_hold, /**< `(I,WO)` Transmit hold interval. */
+	lldpctl_k_config_bond_slave_src_mac_type, /**< `(I,WO)` bond slave src mac type. */
 } lldpctl_key_t;
 
 /**
