@@ -333,6 +333,8 @@ struct lldpd_config {
 	int c_tx_fast_interval;	/* Time intr between sends during fast start */
 #endif
 	int c_tx_hold;		/* Transmit hold */
+	int c_bond_slave_src_mac_type; /* Src mac type in lldp frames over bond
+					  slaves */
 };
 MARSHAL_BEGIN(lldpd_config)
 MARSHAL_STR(lldpd_config, c_mgmt_pattern)
@@ -369,6 +371,7 @@ struct lldpd_hardware {
 	struct lldpd		*h_cfg;	    /* Pointer to main configuration */
 	void			*h_recv;    /* FD for reception */
 	int			 h_sendfd;  /* FD for sending, only used by h_ops */
+	int			 h_mangle;  /* 1 if we have to mangle the MAC address */
 	struct lldpd_ops	*h_ops;	    /* Hardware-dependent functions */
 	void			*h_data;    /* Hardware-dependent data */
 	void			*h_timer;   /* Timer for this port */
@@ -437,7 +440,8 @@ MARSHAL_END(lldpd_neighbor_change);
 void	 lldpd_chassis_mgmt_cleanup(struct lldpd_chassis *);
 void	 lldpd_chassis_cleanup(struct lldpd_chassis *, int);
 void	 lldpd_remote_cleanup(struct lldpd_hardware *,
-    void(*expire)(struct lldpd_hardware *, struct lldpd_port *));
+    void(*expire)(struct lldpd_hardware *, struct lldpd_port *),
+    int);
 void	 lldpd_port_cleanup(struct lldpd_port *, int);
 void	 lldpd_config_cleanup(struct lldpd_config *);
 #ifdef ENABLE_DOT1
