@@ -4,7 +4,6 @@
 #include <arpa/inet.h>
 #include <netinet/in.h>
 #include <check.h>
-#include "../src/lldpd.h"
 #include "common.h"
 
 char filenameprefix[] = "edp_send";
@@ -77,7 +76,7 @@ Extreme Discovery Protocol
 	hardware.h_ifindex = 4;
 	chassis.c_id_subtype = LLDP_CHASSISID_SUBTYPE_LLADDR;
 	chassis.c_id = macaddress;
-	chassis.c_id_len = ETH_ALEN;
+	chassis.c_id_len = ETHER_ADDR_LEN;
 	chassis.c_name = "First chassis";
 	/* Build packet */
 	n = edp_send(NULL, &hardware);
@@ -241,7 +240,7 @@ Extreme Discovery Protocol
 	hardware.h_ifindex = 4;
 	chassis.c_id_subtype = LLDP_CHASSISID_SUBTYPE_LLADDR;
 	chassis.c_id = macaddress;
-	chassis.c_id_len = ETH_ALEN;
+	chassis.c_id_len = ETHER_ADDR_LEN;
 	chassis.c_name = "First chassis";
 	vlan1.v_name = "First VLAN"; vlan1.v_vid = 157;
 	vlan2.v_name = "Second VLAN"; vlan2.v_vid = 1247;
@@ -500,7 +499,7 @@ Extreme Discovery Protocol
 	struct lldpd cfg;
 	char mac1[] = { 0x00, 0x04, 0x96, 0x05, 0x44, 0x6f };
 
-	cfg.g_mgmt_pattern = NULL;
+	cfg.g_config.c_mgmt_pattern = NULL;
 	fail_unless(edp_decode(&cfg, pkt1, sizeof(pkt1), &hardware,
 		&nchassis, &nport) != -1);
 	if (!nchassis || !nport) {
@@ -509,8 +508,8 @@ Extreme Discovery Protocol
 	}
 	ck_assert_int_eq(nchassis->c_id_subtype,
 	    LLDP_CHASSISID_SUBTYPE_LLADDR);
-	ck_assert_int_eq(nchassis->c_id_len, ETH_ALEN);
-	fail_unless(memcmp(nchassis->c_id, mac1, ETH_ALEN) == 0);
+	ck_assert_int_eq(nchassis->c_id_len, ETHER_ADDR_LEN);
+	fail_unless(memcmp(nchassis->c_id, mac1, ETHER_ADDR_LEN) == 0);
 	ck_assert_int_eq(nport->p_id_subtype,
 	    LLDP_PORTID_SUBTYPE_IFNAME);
 	ck_assert_int_eq(nport->p_id_len, strlen("1/1"));
@@ -536,7 +535,7 @@ Extreme Discovery Protocol
 		fail("unable to find our previous port?");
 		return;
 	}
-	ck_assert_int_eq(nport->p_chassis->c_mgmt.s_addr,
+	ck_assert_int_eq(TAILQ_FIRST(&nport->p_chassis->c_mgmt)->m_addr.inet.s_addr,
 	    (u_int32_t)inet_addr("10.50.0.63"));
 	if (TAILQ_EMPTY(&nport->p_vlans)) {
 		fail("no VLAN");
@@ -574,6 +573,8 @@ edp_suite(void)
 
 #ifdef ENABLE_EDP
 	TCase *tc_send = tcase_create("Send EDP packets");
+	TCase *tc_receive = tcase_create("Receive EDP packets");
+
 	tcase_add_checked_fixture(tc_send, pcap_setup, pcap_teardown);
 	tcase_add_test(tc_send, test_send_basic);
 #ifdef ENABLE_DOT1
@@ -581,7 +582,6 @@ edp_suite(void)
 #endif
 	suite_add_tcase(s, tc_send);
 
-	TCase *tc_receive = tcase_create("Receive EDP packets");
 	tcase_add_test(tc_receive, test_recv_edp);
 	suite_add_tcase(s, tc_receive);
 #endif
