@@ -4,21 +4,25 @@
 #include <arpa/inet.h>
 #include <netinet/in.h>
 #include <check.h>
-#include "../src/lldpd.h"
 #include "common.h"
 
 char filenameprefix[] = "lldp_send";
 
+#define ck_assert_str_eq_n(X, Y, N) \
+	ck_assert_msg(!strncmp(X, Y, N), "Assertion '"#X"=="#Y"' failed: "#X"==\"%s\", "#Y"==\"%s\"", X, Y)
+
 static void
 check_received_port(
 	struct lldpd_port *sport,
 	struct lldpd_port *rport)
 {
 	ck_assert_int_eq(rport->p_id_subtype, sport->p_id_subtype);
-	ck_assert_str_eq(rport->p_id, sport->p_id);
 	ck_assert_int_eq(rport->p_id_len, sport->p_id_len);
+	ck_assert_str_eq_n(rport->p_id, sport->p_id, sport->p_id_len);
 	ck_assert_str_eq(rport->p_descr, sport->p_descr);
+#ifdef ENABLE_DOT3
 	ck_assert_int_eq(rport->p_mfs, sport->p_mfs);
+#endif
 	return;
 }
 
@@ -28,8 +32,8 @@ check_received_chassis(
 	struct lldpd_chassis *rchassis)
 {
 	ck_assert_int_eq(rchassis->c_id_subtype, schassis->c_id_subtype);
-	ck_assert_str_eq(rchassis->c_id, schassis->c_id);
 	ck_assert_int_eq(rchassis->c_id_len, schassis->c_id_len);
+	ck_assert_str_eq_n(rchassis->c_id, schassis->c_id, schassis->c_id_len);
 	ck_assert_str_eq(rchassis->c_name, schassis->c_name);
 	ck_assert_str_eq(rchassis->c_descr, schassis->c_descr);
 	ck_assert_int_eq(rchassis->c_cap_available, schassis->c_cap_available);
@@ -46,29 +50,30 @@ check_received_port_med(
 	ck_assert_int_eq(rport->p_med_cap_enabled, sport->p_med_cap_enabled);
 	ck_assert_int_eq(rport->p_med_cap_enabled, sport->p_med_cap_enabled);
 	ck_assert_int_eq(
-		rport->p_med_location[LLDPMED_LOCFORMAT_CIVIC-1].format,
-		sport->p_med_location[LLDPMED_LOCFORMAT_CIVIC-1].format);
+		rport->p_med_location[LLDP_MED_LOCFORMAT_CIVIC-1].format,
+		sport->p_med_location[LLDP_MED_LOCFORMAT_CIVIC-1].format);
 	ck_assert_int_eq(
-		rport->p_med_location[LLDPMED_LOCFORMAT_CIVIC-1].data_len,
-		sport->p_med_location[LLDPMED_LOCFORMAT_CIVIC-1].data_len);
-	ck_assert_str_eq(
-		rport->p_med_location[LLDPMED_LOCFORMAT_CIVIC-1].data,
-		sport->p_med_location[LLDPMED_LOCFORMAT_CIVIC-1].data);
+		rport->p_med_location[LLDP_MED_LOCFORMAT_CIVIC-1].data_len,
+		sport->p_med_location[LLDP_MED_LOCFORMAT_CIVIC-1].data_len);
+	ck_assert_str_eq_n(
+		rport->p_med_location[LLDP_MED_LOCFORMAT_CIVIC-1].data,
+		sport->p_med_location[LLDP_MED_LOCFORMAT_CIVIC-1].data,
+		sport->p_med_location[LLDP_MED_LOCFORMAT_CIVIC-1].data_len);
 	ck_assert_int_eq(
-		rport->p_med_policy[LLDPMED_APPTYPE_SOFTPHONEVOICE-1].type,
-		sport->p_med_policy[LLDPMED_APPTYPE_SOFTPHONEVOICE-1].type);
+		rport->p_med_policy[LLDP_MED_APPTYPE_SOFTPHONEVOICE-1].type,
+		sport->p_med_policy[LLDP_MED_APPTYPE_SOFTPHONEVOICE-1].type);
 	ck_assert_int_eq(
-		rport->p_med_policy[LLDPMED_APPTYPE_SOFTPHONEVOICE-1].tagged,
-		sport->p_med_policy[LLDPMED_APPTYPE_SOFTPHONEVOICE-1].tagged);
+		rport->p_med_policy[LLDP_MED_APPTYPE_SOFTPHONEVOICE-1].tagged,
+		sport->p_med_policy[LLDP_MED_APPTYPE_SOFTPHONEVOICE-1].tagged);
 	ck_assert_int_eq(
-		rport->p_med_policy[LLDPMED_APPTYPE_SOFTPHONEVOICE-1].vid,
-		sport->p_med_policy[LLDPMED_APPTYPE_SOFTPHONEVOICE-1].vid);
+		rport->p_med_policy[LLDP_MED_APPTYPE_SOFTPHONEVOICE-1].vid,
+		sport->p_med_policy[LLDP_MED_APPTYPE_SOFTPHONEVOICE-1].vid);
 	ck_assert_int_eq(
-		rport->p_med_policy[LLDPMED_APPTYPE_SOFTPHONEVOICE-1].priority,
-		sport->p_med_policy[LLDPMED_APPTYPE_SOFTPHONEVOICE-1].priority);
+		rport->p_med_policy[LLDP_MED_APPTYPE_SOFTPHONEVOICE-1].priority,
+		sport->p_med_policy[LLDP_MED_APPTYPE_SOFTPHONEVOICE-1].priority);
 	ck_assert_int_eq(
-		rport->p_med_policy[LLDPMED_APPTYPE_SOFTPHONEVOICE-1].dscp,
-		sport->p_med_policy[LLDPMED_APPTYPE_SOFTPHONEVOICE-1].dscp);
+		rport->p_med_policy[LLDP_MED_APPTYPE_SOFTPHONEVOICE-1].dscp,
+		sport->p_med_policy[LLDP_MED_APPTYPE_SOFTPHONEVOICE-1].dscp);
 	ck_assert_int_eq(
 		rport->p_med_power.devicetype, sport->p_med_power.devicetype);
 	ck_assert_int_eq(rport->p_med_power.source, sport->p_med_power.source);
@@ -127,7 +132,7 @@ START_TEST (test_send_rcv_basic)
 	hardware.h_lport.p_mfs = 1516;
 	chassis.c_id_subtype = LLDP_CHASSISID_SUBTYPE_LLADDR;
 	chassis.c_id = macaddress;
-	chassis.c_id_len = ETH_ALEN;
+	chassis.c_id_len = ETHER_ADDR_LEN;
 	chassis.c_name = "First chassis";
 	chassis.c_descr = "Chassis description";
 	chassis.c_cap_available = chassis.c_cap_enabled = LLDP_CAP_ROUTER;
@@ -176,7 +181,7 @@ START_TEST (test_send_rcv_dot1_tlvs)
 	/* Populate port and chassis */
 	hardware.h_lport.p_id_subtype = LLDP_PORTID_SUBTYPE_LLADDR;
 	hardware.h_lport.p_id = macaddress;
-	hardware.h_lport.p_id_len = ETH_ALEN;
+	hardware.h_lport.p_id_len = ETHER_ADDR_LEN;
 	hardware.h_lport.p_descr = "Fake port description";
 	hardware.h_lport.p_mfs = 1516;
 	hardware.h_lport.p_pvid = 1500;
@@ -272,14 +277,16 @@ START_TEST (test_send_rcv_dot1_tlvs)
 		return;
 	}
 	rpi = TAILQ_FIRST(&nport->p_pids);
-	ck_assert_str_eq(rpi->p_pi, pi1.p_pi);
+	ck_assert_int_eq(rpi->p_pi_len, pi1.p_pi_len);
+	ck_assert_str_eq_n(rpi->p_pi, pi1.p_pi, pi1.p_pi_len);
 
 	rpi = TAILQ_NEXT(rpi, p_entries);
 	if (!rpi) {
 		fail("no more Protocol Identity TLVs");
 		return;
 	}
-	ck_assert_str_eq(rpi->p_pi, pi2.p_pi);
+	ck_assert_int_eq(rpi->p_pi_len, pi2.p_pi_len);
+	ck_assert_str_eq_n(rpi->p_pi, pi2.p_pi, pi2.p_pi_len);
 
 	rpi = TAILQ_NEXT(rpi, p_entries);
 	fail_unless(rpi == NULL);
@@ -300,7 +307,7 @@ START_TEST (test_send_rcv_med)
 	/* Populate port and chassis */
 	hardware.h_lport.p_id_subtype = LLDP_PORTID_SUBTYPE_LLADDR;
 	hardware.h_lport.p_id = macaddress;
-	hardware.h_lport.p_id_len = ETH_ALEN;
+	hardware.h_lport.p_id_len = ETHER_ADDR_LEN;
 	hardware.h_lport.p_descr = "Fake port description";
 	hardware.h_lport.p_mfs = 1516;
 	chassis.c_id_subtype = LLDP_CHASSISID_SUBTYPE_LOCAL;
@@ -310,33 +317,33 @@ START_TEST (test_send_rcv_med)
 	chassis.c_descr = "Chassis description";
 	chassis.c_cap_available = LLDP_CAP_ROUTER | LLDP_CAP_BRIDGE;
 	chassis.c_cap_enabled = LLDP_CAP_ROUTER;
-	chassis.c_med_cap_available = LLDPMED_CAP_CAP | LLDPMED_CAP_POLICY |
-		LLDPMED_CAP_LOCATION | LLDPMED_CAP_MDI_PSE |
-		LLDPMED_CAP_IV;
-	chassis.c_med_type = LLDPMED_CLASS_III;
+	chassis.c_med_cap_available = LLDP_MED_CAP_CAP | LLDP_MED_CAP_POLICY |
+		LLDP_MED_CAP_LOCATION | LLDP_MED_CAP_MDI_PSE |
+		LLDP_MED_CAP_IV;
+	chassis.c_med_type = LLDP_MED_CLASS_III;
 	chassis.c_med_hw = "hardware rev 5";
 	chassis.c_med_fw = "47b5";
 	chassis.c_med_sw = "2.6.22b5";
 	chassis.c_med_sn = "SN 47842";
 	hardware.h_lport.p_med_cap_enabled = chassis.c_med_cap_available;
-	hardware.h_lport.p_med_location[LLDPMED_LOCFORMAT_CIVIC-1].format =
-		LLDPMED_LOCFORMAT_CIVIC;
-	hardware.h_lport.p_med_location[LLDPMED_LOCFORMAT_CIVIC-1].data = "Your favorite city";
-	hardware.h_lport.p_med_location[LLDPMED_LOCFORMAT_CIVIC-1].data_len = 
+	hardware.h_lport.p_med_location[LLDP_MED_LOCFORMAT_CIVIC-1].format =
+		LLDP_MED_LOCFORMAT_CIVIC;
+	hardware.h_lport.p_med_location[LLDP_MED_LOCFORMAT_CIVIC-1].data = "Your favorite city";
+	hardware.h_lport.p_med_location[LLDP_MED_LOCFORMAT_CIVIC-1].data_len = 
 		sizeof("Your favorite city");
-	hardware.h_lport.p_med_policy[LLDPMED_APPTYPE_SOFTPHONEVOICE-1].type =
-		LLDPMED_APPTYPE_SOFTPHONEVOICE;
-	hardware.h_lport.p_med_policy[LLDPMED_APPTYPE_SOFTPHONEVOICE-1].tagged =
+	hardware.h_lport.p_med_policy[LLDP_MED_APPTYPE_SOFTPHONEVOICE-1].type =
+		LLDP_MED_APPTYPE_SOFTPHONEVOICE;
+	hardware.h_lport.p_med_policy[LLDP_MED_APPTYPE_SOFTPHONEVOICE-1].tagged =
 		1;
-	hardware.h_lport.p_med_policy[LLDPMED_APPTYPE_SOFTPHONEVOICE-1].vid =
+	hardware.h_lport.p_med_policy[LLDP_MED_APPTYPE_SOFTPHONEVOICE-1].vid =
 		51;
-	hardware.h_lport.p_med_policy[LLDPMED_APPTYPE_SOFTPHONEVOICE-1].priority =
+	hardware.h_lport.p_med_policy[LLDP_MED_APPTYPE_SOFTPHONEVOICE-1].priority =
 		6;
-	hardware.h_lport.p_med_policy[LLDPMED_APPTYPE_SOFTPHONEVOICE-1].dscp =
+	hardware.h_lport.p_med_policy[LLDP_MED_APPTYPE_SOFTPHONEVOICE-1].dscp =
 		46;
-	hardware.h_lport.p_med_power.devicetype = LLDPMED_POW_TYPE_PSE;
-	hardware.h_lport.p_med_power.source = LLDPMED_POW_SOURCE_PRIMARY;
-	hardware.h_lport.p_med_power.priority = LLDPMED_POW_PRIO_HIGH;
+	hardware.h_lport.p_med_power.devicetype = LLDP_MED_POW_TYPE_PSE;
+	hardware.h_lport.p_med_power.source = LLDP_MED_POW_SOURCE_PRIMARY;
+	hardware.h_lport.p_med_power.priority = LLDP_MED_POW_PRIO_HIGH;
 	hardware.h_lport.p_med_power.val = 65;
 
 	/* Build packet */
@@ -394,7 +401,7 @@ START_TEST (test_send_rcv_dot3)
 	hardware.h_lport.p_macphy.mau_type = LLDP_DOT3_MAU_100BASETXFD;
 	chassis.c_id_subtype = LLDP_CHASSISID_SUBTYPE_LLADDR;
 	chassis.c_id = macaddress;
-	chassis.c_id_len = ETH_ALEN;
+	chassis.c_id_len = ETHER_ADDR_LEN;
 	chassis.c_name = "Fourth chassis";
 	chassis.c_descr = "Long chassis description";
 	chassis.c_cap_available = chassis.c_cap_enabled = LLDP_CAP_ROUTER | LLDP_CAP_WLAN;
@@ -477,12 +484,12 @@ Link Layer Discovery Protocol
 	}
 	ck_assert_int_eq(nchassis->c_id_subtype,
 	    LLDP_CHASSISID_SUBTYPE_LLADDR);
-	ck_assert_int_eq(nchassis->c_id_len, ETH_ALEN);
-	fail_unless(memcmp(mac1, nchassis->c_id, ETH_ALEN) == 0);
+	ck_assert_int_eq(nchassis->c_id_len, ETHER_ADDR_LEN);
+	fail_unless(memcmp(mac1, nchassis->c_id, ETHER_ADDR_LEN) == 0);
 	ck_assert_int_eq(nport->p_id_subtype,
 	    LLDP_PORTID_SUBTYPE_LLADDR);
-	ck_assert_int_eq(nport->p_id_len, ETH_ALEN);
-	fail_unless(memcmp(mac2, nport->p_id, ETH_ALEN) == 0);
+	ck_assert_int_eq(nport->p_id_len, ETHER_ADDR_LEN);
+	fail_unless(memcmp(mac2, nport->p_id, ETHER_ADDR_LEN) == 0);
 	ck_assert_int_eq(nchassis->c_ttl, 120);
 	ck_assert_str_eq(nchassis->c_name, "Not received");
 	ck_assert_str_eq(nchassis->c_descr, "Not received");
@@ -682,12 +689,12 @@ Link Layer Discovery Protocol
 	}
 	ck_assert_int_eq(nchassis->c_id_subtype,
 	    LLDP_CHASSISID_SUBTYPE_LLADDR);
-	ck_assert_int_eq(nchassis->c_id_len, ETH_ALEN);
-	fail_unless(memcmp(mac1, nchassis->c_id, ETH_ALEN) == 0);
+	ck_assert_int_eq(nchassis->c_id_len, ETHER_ADDR_LEN);
+	fail_unless(memcmp(mac1, nchassis->c_id, ETHER_ADDR_LEN) == 0);
 	ck_assert_int_eq(nport->p_id_subtype,
 	    LLDP_PORTID_SUBTYPE_LLADDR);
-	ck_assert_int_eq(nport->p_id_len, ETH_ALEN);
-	fail_unless(memcmp(mac1, nport->p_id, ETH_ALEN) == 0);
+	ck_assert_int_eq(nport->p_id_len, ETHER_ADDR_LEN);
+	fail_unless(memcmp(mac1, nport->p_id, ETHER_ADDR_LEN) == 0);
 	ck_assert_int_eq(nchassis->c_ttl, 120);
 	ck_assert_str_eq(nchassis->c_name, "naruto.XXXXXXXXXXXXXXXXXXX");
 	ck_assert_str_eq(nchassis->c_descr,
@@ -697,9 +704,9 @@ Link Layer Discovery Protocol
 	    LLDP_CAP_WLAN | LLDP_CAP_ROUTER | LLDP_CAP_BRIDGE);
 	ck_assert_int_eq(nchassis->c_cap_enabled,
 	    LLDP_CAP_ROUTER | LLDP_CAP_BRIDGE);
-	ck_assert_int_eq(nchassis->c_mgmt.s_addr,
+	ck_assert_int_eq(nchassis->c_mgmt.tqh_first->m_addr.inet.s_addr,
 	    (u_int32_t)inet_addr("10.238.80.75"));
-	ck_assert_int_eq(nchassis->c_mgmt_if, 3);
+	ck_assert_int_eq(nchassis->c_mgmt.tqh_first->m_iface, 3);
 #ifdef ENABLE_DOT3
 	ck_assert_int_eq(nport->p_aggregid, 0);
 	ck_assert_int_eq(nport->p_macphy.autoneg_enabled, 1);
@@ -732,6 +739,8 @@ Suite *
 lldp_suite(void)
 {
 	Suite *s = suite_create("LLDP");
+	TCase *tc_send = tcase_create("Send LLDP packets");
+	TCase *tc_receive = tcase_create("Receive LLDP packets");
 
 	/* Send tests are first run without knowing the result. The
 	   result is then checked with:
@@ -742,7 +751,6 @@ lldp_suite(void)
 	   be able to do this).
 	*/
 
-	TCase *tc_send = tcase_create("Send LLDP packets");
 	tcase_add_checked_fixture(tc_send, pcap_setup, pcap_teardown);
 	tcase_add_test(tc_send, test_send_rcv_basic);
 #ifdef ENABLE_DOT1
@@ -756,7 +764,6 @@ lldp_suite(void)
 #endif
 	suite_add_tcase(s, tc_send);
 
-	TCase *tc_receive = tcase_create("Receive LLDP packets");
 	tcase_add_test(tc_receive, test_recv_min);
 	tcase_add_test(tc_receive, test_recv_lldpd);
 	suite_add_tcase(s, tc_receive);
